##############################################################################
#
# JobListings for Zope
#
# Copyright (c) Mark Gibson  All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Mark Gibson      mngibso@excite.com
#
##########################################################################
import os
import Globals
import string
from AccessControl import ClassSecurityInfo
from OFS.DTMLMethod import DTMLMethod
from Globals import DTMLFile
from OFS.Folder import Folder
from OFS.Image import Image
from Products.ZCatalog.ZCatalog import ZCatalog
from Products.ZCatalog.Vocabulary import Vocabulary
from Products.PythonScripts import PythonScript
from JobEntry import JobEntry
import whrandom
from AccessControl import SecurityManagement

__doc__ = """
   JobListings Product
      JobListings is designed for use with Zope.  http://zope.org

   file: JobListings.py

   JobListings.py is a folderish class that contains dtml methods for
   managing JobEntry objects.

"""
__version__ = '0.1'

manage_addJobListingsForm = DTMLFile('dtml/addJobListings',globals())

def manage_addJobListings(self, id, title='', catalog='', vocabulary='',  REQUEST=None):

   """ """
   jobListings=JobListings(id, title, catalog, vocabulary, self, REQUEST) 
   self._setObject(id, jobListings)
   if REQUEST is not None:
      return self.manage_main(self, REQUEST, update_menu=1)


class JobListings(Folder):
   """ Job Folder """

   status_list=JobEntry.status_list

   meta_type = 'Job Listings'

   _properties = (
        { 'id': 'title', 'type': 'string', 'mode': 'w' },
        { 'id': 'catalog_id', 'type': 'string', 'mode': 'w'},
    )

   #use this to create state dropdowns
   state_names_list = (
         ("AL","Alabama"),
         ("AK","Alaska"),
         ("AZ","Arizona"),
         ("AR","Arkansas"),
         ("CA","California"),
         ("CO","Colorado"),
         ("CT","Connecticut"),
         ("DE","Delaware"),
         ("DC","District of Columbia"),
         ("FL","Florida"),
         ("GA","Georgia"),
         ("GU","Guam"),
         ("HI","Hawaii"),
         ("ID","Idaho"),
         ("IL","Illinois"),
         ("IN","Indiana"),
         ("IA","Iowa"),
         ("KS","Kansas"),
         ("KY","Kentucky"),
         ("LA","Louisiana"),
         ("ME","Maine"),
         ("MD","Maryland"),
         ("MA","Massachusetts"),
         ("MI","Michigan"),
         ("MN","Minnesota"),
         ("MS","Mississippi"),
         ("MO","Missouri"),
         ("MT","Montana"),
         ("NE","Nebraska"),
         ("NV","Nevada"),
         ("NH","New Hampshire"),
         ("NJ","New Jersey"),
         ("NM","New Mexico"),
         ("NY","New York"),
         ("NC","North Carolina"),
         ("ND","North Dakota"),
         ("OH","Ohio"),
         ("OK","Oklahoma"),
         ("OR","Oregon"),
         ("PA","Pennsylvania"),
         ("PR","Puerto Rico"),
         ("RI","Rhode Island"),
         ("SC","South Carolina"),
         ("SD","South Dakota"),
         ("TN","Tennessee"),
         ("TX","Texas"),
         ("UT","Utah"),
         ("VT","Vermont"),
         ("VA","Virginia"),
         ("VI","US Virgin Islands"),
         ("WA","Washington"),
         ("WV","West Virginia"),
         ("WI","Wisconsin"),
         ("WY","Wyoming"),
   )

   security=ClassSecurityInfo()
   def __init__(self, id, title='', catalog_id='', vocab_id='', 
                 container=None, REQUEST=None):
      """ Job Listings Constructor """
      self.id=id
      self.title=title
      self.catalog_id=catalog_id

      if vocab_id:
         if container:
            self = self.__of__(container)
         else:
            raise AttributeError, ("You cannot specify a vocab_id without "
                                      "also specifying a container.")
      else:
         # Add a global vocabulary
         vocab_id = 'Vocabulary'
         ob = Vocabulary(vocab_id, '', globbing=1)
         self._setObject(vocab_id, ob)

      if not self.catalog_id:
         #No catalog specified, so add one
         catalog=ZCatalog('Catalog','Job Catalog', vocab_id, self)
         self.catalog_id='Catalog'
         self._setObject(self.catalog_id,catalog) 
            
      # add indexes and columns to catalog
      self.prepareCatalog() 
      self._addMethods() 

   def _addMethods(self):
      """ add images, python scripts, and dtml methods to the folder """
      path=os.path.join(Globals.package_home(globals()), 'instance')
      for id in os.listdir(path):
         file = open(os.path.join(path, id))
         try: name,ext=string.split(id,'.')
         except: continue
         #create python scripts
         if ext == 'pys':
            ob=PythonScript.PythonScript(name)
            ob.write(file.read())
            self._setObject(name,ob)
            file.close()
            #re-assign bindings.  Bindings don't 'take' unless I do this.
            ob.ZBindings_edit(ob.getBindingAssignments().getAssignedNames())
         #create dtml methods
         elif ext=='dtml':
            ob=DTMLMethod(file.read(),__name__=name)
            self._setObject(name,ob)
            file.close()
         #create images
         elif ext in ('gif','jpg'): 
            id=name+'.'+ext
            #only one image now, so we can set title to 'back'
            ob=Image(id,'back',file)
            self._setObject(name+'.' + ext,ob)
            file.close()
      
   security.declareProtected('Add Job Listings', 'prepareCatalog')
   def prepareCatalog(self):
      """ Add indexes and columns to help manage jobs """
      catalog=getattr(self,self.catalog_id) 
      indexes=catalog.indexes()
      columns=catalog.schema()

      if not 'meta_type' in indexes:
         catalog.addIndex('meta_type','FieldIndex')

      if not 'id' in indexes:
         catalog.addIndex('id','FieldIndex')
      if not 'id' in columns:
         catalog.addColumn('id')

      if not 'title' in indexes:
         catalog.addIndex('title','TextIndex')
      if not 'title' in columns:
         catalog.addColumn('title')

      if not 'publishDate' in indexes:
         catalog.addIndex('publishDate','FieldIndex')
      if not 'publishDate' in columns:
         catalog.addColumn('publishDate')

      if not 'jobType' in indexes:
         catalog.addIndex('jobType','KeywordIndex')

      if not 'jobState' in indexes:
         catalog.addIndex('jobState','FieldIndex')
      if not 'jobState' in columns:
         catalog.addColumn('jobState')

      if not 'jobCity' in columns:
         catalog.addColumn('jobCity')

      if not 'deadlineDate' in indexes:
         catalog.addIndex('deadlineDate','FieldIndex')
      if not 'jobState' in columns:
         catalog.addColumn('deadlineDate')

      if not 'validDate' in indexes:
         catalog.addIndex('validDate','FieldIndex')
      if not 'validDate' in columns:
         catalog.addColumn('validDate')

      if not 'expirationDate' in indexes:
         catalog.addIndex('expirationDate','FieldIndex')
      if not 'expirationDate' in columns:
         catalog.addColumn('expirationDate')

      if not 'status' in indexes:
         catalog.addIndex('status','FieldIndex')
      if not 'status' in columns:
         catalog.addColumn('status')

      if not 'organization' in columns:
         catalog.addColumn('organization')


   security.declareProtected('Change Job Entry', 'editJobEntry')
   def editJobEntry(self, context, REQUEST=None):
      """ """
      if REQUEST and REQUEST.get('publish',''):
         context.changeProperties({'status':'Published'})
         msg="Job+Published"
         return REQUEST.RESPONSE.redirect(self.absolute_url() + '?msg=%s'%msg)
      [error,errors] = self.validateFields()
      if error:
         if REQUEST is not None:
            REQUEST.set('errors',errors)
            return self.editJobForm(self,REQUEST)
      else:
         context.changeProperties(REQUEST)
      msg="Job+Changed"
      return REQUEST.RESPONSE.redirect(self.absolute_url() + '?msg=%s'%msg)

   security.declareProtected('View', 'getStorageFolder')
   def getStorageFolder(self):
      """ Return the folder that holds job entries 
          
          The storage folder is either self, or as specified in the
          job_store property.
      """
      if hasattr(self,'job_store') and self.job_store:
         # 'job_store' is of the form 'foo/bar' so we need to walk the
         # path
         dirs = string.split(self.job_store, '/')
         storage=getattr(self, dirs[0])
         for d in dirs:
            storage=getattr(storage, d)
      else: storage=self

      return storage

   security.declareProtected('Add Job Entry', 'addJobEntry')
   def addJobEntry(self, id='', title='', REQUEST=None):
      """ Create a new Job Entry """
      #validateFields should be a python script in the Job Listings folder
      [error,errors] = self.validateFields()
      if error:
         if REQUEST is not None:
            REQUEST.set('errors',errors)
            return self.addJobForm(self,REQUEST)

      if REQUEST and string.strip(REQUEST.get('addJob','')) == 'Re-Edit':
         return self.addJobForm(self,REQUEST)
      elif REQUEST and string.strip(REQUEST.get('addJob','')) == 'Preview':
         return self.jobPreview(self,REQUEST)

      #look for alternative storage folder
      storage=self.getStorageFolder()

      #set the id if we don't alread have one
      if not id:
         id=self.genJobId(storage) 
      #set status
      if SecurityManagement.getSecurityManager().getUser().has_permission('Add Job Entry Without Review',storage):
         status='Published'
      else:
         status='Private'

      job=JobEntry(id, title, status, REQUEST)
      job.stateTransition(status, force=1)
      job.manage_editCataloger(self.catalog_id)
      job.changeProperties(REQUEST)
      storage._setObject(job.getId(),job)
      #set the catalog
      job.reindex_object()


      if REQUEST is not None:
         msg='Job+has+been+added.'
         return REQUEST.RESPONSE.redirect(self.absolute_url() + '?msg=%s'%msg)
            
  

   security.declareProtected('Delete objects', 'delete')
   def delete(self,REQUEST):
     """ Delete the job entry.
           First, look in self,
           Then look in the storage folder 
     """
     storage=self.getStorageFolder()
     if REQUEST.id in self.objectIds():
        self.manage_delObjects(REQUEST.id)
        msg='Job+Deleted'
     elif storage != self:
        if REQUEST.id in storage.objectIds():
           storage.manage_delObjects(REQUEST.id)
           msg='Job+Deleted'
        else:
           msg='Job+could+not+be+found'
     else:
           msg='Job+could+not+be+found'
     return REQUEST.RESPONSE.redirect(self.absolute_url() + '?msg=%s'%msg)

   def genJobId(self, folder=None, prefix='Job_'):
      """ Generate a unique, random id """
      def getNewId():
         s=string.lowercase + string.digits
         newid=''
         #suffix is 5 characters long
         for x in range(5):
            newid=newid+whrandom.choice(s)
         return newid

      if not folder:
         folder=self.getStorageFolder()
   
      #loop until we get a unique id
      objIds=folder.objectIds()
      while 1:
         newid=prefix + getNewId()
         if newid not in objIds:
            break 
   
      return newid
      
